@@ -31,6 +31,7 @@ import java.util.Vector;
 
 import org.apache.log4j.Logger;
 
+import com.verisign.epp.codec.allocationtoken.EPPAllocationToken;
 import com.verisign.epp.codec.coaext.EPPCoaExtAttr;
 import com.verisign.epp.codec.coaext.EPPCoaExtCreate;
 import com.verisign.epp.codec.coaext.EPPCoaExtInfData;
@@ -64,6 +65,7 @@ import com.verisign.epp.codec.gen.EPPService;
 import com.verisign.epp.codec.gen.EPPTransId;
 import com.verisign.epp.codec.gen.EPPUtil;
 import com.verisign.epp.codec.idnext.EPPIdnLangTag;
+import com.verisign.epp.codec.idnmap.EPPIdnMap;
 import com.verisign.epp.codec.launch.EPPLaunchCheck;
 import com.verisign.epp.codec.launch.EPPLaunchCreate;
 import com.verisign.epp.codec.launch.EPPLaunchDelete;
@@ -141,14 +143,38 @@ public class NSDomainHandler extends EPPDomainHandler {
 	/** Sub-handler for handling version 1.1 extension messages. */
 	SecDNSV11SubDomainHandler v11SubHandler = new SecDNSV11SubDomainHandler();
 
-	/** Personal Registration Domain Handler used for the .name create, transfer, and info commands. */
+	/**
+	 * Personal Registration Domain Handler used for the .name create, transfer,
+	 * and info commands.
+	 */
 	PersRegDomainHandler pergRegDomainHandler = new PersRegDomainHandler();
-	
+
 	/** Launch Phase Domain Handler used when the launch extension is passed. */
 	LaunchDomainHandler launchDomainHandler = new LaunchDomainHandler();
 
 	/** Related Domain Handler used when the related domain extension is passed. */
 	RelatedDomainHandler relatedDomainHandler = new RelatedDomainHandler();
+
+	/**
+	 * Allocation Token Domain Handler used when the allocation token extension
+	 * is passed.
+	 */
+	AllocationTokenDomainHandler allocationTokenDomainHandler = new AllocationTokenDomainHandler();
+
+	/**
+	 * Fee Domain Handler used with the Fee Extension.
+	 */
+	FeeDomainHandler feeDomainHandler = new FeeDomainHandler();
+
+	/**
+	 * Change Poll Domain Handler used with the Change Poll Extension.
+	 */
+	ChangePollDomainHandler changePollDomainHandler = new ChangePollDomainHandler();
+
+	/**
+	 * IDN Map Domain Handler used with the Idn Map Extension.
+	 */
+	IdnMapDomainHandler idnMapDomainHandler = new IdnMapDomainHandler();
 	
 	/**
 	 * Constructs an instance of NSDomainHandler
@@ -273,27 +299,57 @@ public class NSDomainHandler extends EPPDomainHandler {
 	 */
 	protected EPPEventResponse doDomainCheck(EPPEvent aEvent, Object aData) {
 		EPPDomainCheckCmd theMessage = (EPPDomainCheckCmd) aEvent.getMessage();
-		
+
 		cat.debug("doDomainCheck: command = [" + theMessage + "]");
-		
-		
+
 		EPPResponse theResponse;
-		
+
 		// Claims Check Command?
 		if (theMessage.hasExtension(EPPLaunchCheck.class)) {
-			EPPEventResponse launchResp = launchDomainHandler.doDomainCheck(aEvent, aData);
-			
+			EPPEventResponse launchResp = launchDomainHandler.doDomainCheck(
+					aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) launchResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return launchResp;
 		}
-		
-		
+
+		// Allocation Token Check?
+		if (theMessage.hasExtension(EPPAllocationToken.class)) {
+			EPPEventResponse allocationTokenResp = allocationTokenDomainHandler
+					.doDomainCheck(aEvent, aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) allocationTokenResp.getResponse())
+						.addExtension(theMessage
+								.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return allocationTokenResp;
+		}
+
+		// Fee Check?
+		if (theMessage
+				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeCheck.class)
+				|| theMessage
+						.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeCheck.class)) {
+			EPPEventResponse feeResp = feeDomainHandler.doDomainCheck(aEvent,
+					aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) feeResp.getResponse()).addExtension(theMessage
+						.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return feeResp;
+		}
 
 		// This is just a vector of strings representing Domain Names
 		Vector vNames = theMessage.getNames();
@@ -374,25 +430,43 @@ public class NSDomainHandler extends EPPDomainHandler {
 	 */
 	protected EPPEventResponse doDomainRenew(EPPEvent aEvent, Object aData) {
 		EPPDomainRenewCmd theMessage = (EPPDomainRenewCmd) aEvent.getMessage();
-		
+
 		cat.debug("doDomainRenew: command = [" + theMessage + "]");
-		
+
 		EPPResponse theResponse;
-		
-		// Related Domain Renew?  
+
+		// Related Domain Renew?
 		if (theMessage.hasExtension(EPPRelatedDomainExtRenew.class)) {
-			EPPEventResponse relatedResp = relatedDomainHandler.doDomainRenew(aEvent, aData);
-			
+			EPPEventResponse relatedResp = relatedDomainHandler.doDomainRenew(
+					aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) relatedResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return relatedResp;
 		}
 
+		// Fee Renew?
+		if (theMessage
+				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeRenew.class)
+				|| theMessage
+						.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeRenew.class)) {
+			EPPEventResponse feeResp = feeDomainHandler.doDomainRenew(aEvent,
+					aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) feeResp.getResponse()).addExtension(theMessage
+						.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return feeResp;
+		}
+
 		/**
 		 * Create the transId for the response with the client trans id and the
 		 * server trans id.
@@ -434,38 +508,52 @@ public class NSDomainHandler extends EPPDomainHandler {
 	protected EPPEventResponse doDomainDelete(EPPEvent aEvent, Object aData) {
 		EPPDomainDeleteCmd theMessage = (EPPDomainDeleteCmd) aEvent
 				.getMessage();
-		
+
 		cat.debug("doDomainDelete: command = [" + theMessage + "]");
-				
-		
+
 		// Launch Domain Delete Command?
 		if (theMessage.hasExtension(EPPLaunchDelete.class)) {
-			EPPEventResponse launchResp = launchDomainHandler.doDomainDelete(aEvent, aData);
-			
+			EPPEventResponse launchResp = launchDomainHandler.doDomainDelete(
+					aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) launchResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return launchResp;
 		}
-		
-		// Related Domain Delete?  
+
+		// Related Domain Delete?
 		if (theMessage.hasExtension(EPPRelatedDomainExtDelete.class)) {
-			EPPEventResponse relatedResp = relatedDomainHandler.doDomainDelete(aEvent, aData);
-			
+			EPPEventResponse relatedResp = relatedDomainHandler.doDomainDelete(
+					aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) relatedResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return relatedResp;
 		}
 
+		// Fee Delete based on the use of the "delete-credit.com" domain name?
+		if (theMessage.getName().equalsIgnoreCase("delete-credit.com")) {
+			EPPEventResponse feeResp = feeDomainHandler.doDomainDelete(aEvent,
+					aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) feeResp.getResponse()).addExtension(theMessage
+						.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return feeResp;
+		}
 
 		/**
 		 * Create the transId for the response with the client trans id and the
@@ -501,53 +589,118 @@ public class NSDomainHandler extends EPPDomainHandler {
 	protected EPPEventResponse doDomainCreate(EPPEvent aEvent, Object aData) {
 		EPPDomainCreateCmd theMessage = (EPPDomainCreateCmd) aEvent
 				.getMessage();
-		
+
 		cat.debug("doDomainCreate: command = [" + theMessage + "]");
-		
-	
+
 		// Launch Domain Create Command?
 		if (theMessage.hasExtension(EPPLaunchCreate.class)) {
-			EPPEventResponse launchResp = launchDomainHandler.doDomainCreate(aEvent, aData);
-			
+			EPPEventResponse launchResp = launchDomainHandler.doDomainCreate(
+					aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) launchResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return launchResp;
 		}
-		
-		// Related Domain Create?  
+
+		// Related Domain Create?
 		if (theMessage.hasExtension(EPPRelatedDomainExtCreate.class)) {
-			EPPEventResponse relatedResp = relatedDomainHandler.doDomainCreate(aEvent, aData);
-			
+			EPPEventResponse relatedResp = relatedDomainHandler.doDomainCreate(
+					aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) relatedResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return relatedResp;
 		}
 
+		// Allocation Token Check?
+		if (theMessage.hasExtension(EPPAllocationToken.class)) {
+			EPPEventResponse allocationTokenResp = allocationTokenDomainHandler
+					.doDomainCreate(aEvent, aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) allocationTokenResp.getResponse())
+						.addExtension(theMessage
+								.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return allocationTokenResp;
+		}
+
+		// Fee Create?
+		if (theMessage
+				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeCreate.class)
+				|| theMessage
+						.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeCreate.class)) {
+			EPPEventResponse feeResp = feeDomainHandler.doDomainCreate(aEvent,
+					aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) feeResp.getResponse()).addExtension(theMessage
+						.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return feeResp;
+		}
+
+		// Create to produce a Change Poll message?
+		if (theMessage.getName().equalsIgnoreCase("change-poll.tld")) {
+			EPPEventResponse changePollResp = changePollDomainHandler
+					.doDomainCreate(aEvent, aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) changePollResp.getResponse())
+						.addExtension(theMessage
+								.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return changePollResp;
+		}
 		
+		// EPP Map?
+		if (theMessage
+				.hasExtension(EPPIdnMap.class)
+				|| theMessage
+						.hasExtension(EPPIdnMap.class)) {
+			EPPEventResponse idnMapResp = idnMapDomainHandler.doDomainCreate(aEvent,
+					aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) idnMapResp.getResponse()).addExtension(theMessage
+						.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return idnMapResp;
+		}
+		
+
 		// .name domain?
 		if (theMessage.getName().endsWith(".name")) {
-			EPPEventResponse persRegResp = pergRegDomainHandler.doDomainCreate(aEvent, aData);
-			
+			EPPEventResponse persRegResp = pergRegDomainHandler.doDomainCreate(
+					aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) persRegResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return persRegResp;
 		}
-		
 
 		EPPTransId transId = new EPPTransId(theMessage.getTransId(), svrTransId);
 
@@ -772,18 +925,21 @@ public class NSDomainHandler extends EPPDomainHandler {
 				return new EPPEventResponse(theResponse);
 			}
 
-		} 
+		}
 		else {
 			theResponse.setResult(EPPResult.SUCCESS);
 		}
-		
-		
-		if (theMessage.getName().startsWith("ERRORTESTBUNDLE")  && theMessage.getName().endsWith(".NAME")) {
-			 theResponse.setResult(EPPResult.ASSOC_PROHIBITS_OP);
-			 theResponse.addExtension(new EPPPersRegCreateErrData(EPPPersRegCreateErrData.ERROR_DEFREG_EXISTS));
-		}else if (theMessage.getName().startsWith("TESTBUNDLE ")  && theMessage.getName().endsWith(".NAME")) {
-			 theResponse.addExtension(new EPPPersRegCreateData(true));
-			
+
+		if (theMessage.getName().startsWith("ERRORTESTBUNDLE")
+				&& theMessage.getName().endsWith(".NAME")) {
+			theResponse.setResult(EPPResult.ASSOC_PROHIBITS_OP);
+			theResponse.addExtension(new EPPPersRegCreateErrData(
+					EPPPersRegCreateErrData.ERROR_DEFREG_EXISTS));
+		}
+		else if (theMessage.getName().startsWith("TESTBUNDLE ")
+				&& theMessage.getName().endsWith(".NAME")) {
+			theResponse.addExtension(new EPPPersRegCreateData(true));
+
 		}
 		// Mirror NameStore Extension in response
 		if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
@@ -809,24 +965,58 @@ public class NSDomainHandler extends EPPDomainHandler {
 	protected EPPEventResponse doDomainTransfer(EPPEvent aEvent, Object aData) {
 		EPPDomainTransferCmd theMessage = (EPPDomainTransferCmd) aEvent
 				.getMessage();
-		
+
 		cat.debug("doDomainTransfer: command = [" + theMessage + "]");
-				
-		// Related Domain Transfer?  
+
+		// Related Domain Transfer?
 		if (theMessage.hasExtension(EPPRelatedDomainExtTransfer.class)) {
-			EPPEventResponse relatedResp = relatedDomainHandler.doDomainTransfer(aEvent, aData);
-			
+			EPPEventResponse relatedResp = relatedDomainHandler
+					.doDomainTransfer(aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) relatedResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return relatedResp;
 		}
-		
-		
+
+		// Allocation Token Transfer?
+		if (theMessage.hasExtension(EPPAllocationToken.class)) {
+			EPPEventResponse allocationTokenResp = allocationTokenDomainHandler
+					.doDomainTransfer(aEvent, aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) allocationTokenResp.getResponse())
+						.addExtension(theMessage
+								.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return allocationTokenResp;
+		}
+
+		// Fee Transfer Request?
+		if (theMessage
+				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeTransfer.class)
+				|| theMessage
+						.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeTransfer.class)
+				|| theMessage.getName().equalsIgnoreCase(
+						"fee-transfer-query.com")) {
+			EPPEventResponse feeResp = feeDomainHandler.doDomainTransfer(
+					aEvent, aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) feeResp.getResponse()).addExtension(theMessage
+						.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return feeResp;
+		}
+
 		// .name domain?
 		if (theMessage.getName().endsWith(".name")) {
 			EPPEventResponse persRegResp = pergRegDomainHandler
@@ -838,7 +1028,7 @@ public class NSDomainHandler extends EPPDomainHandler {
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return persRegResp;
 		}
 
@@ -924,35 +1114,68 @@ public class NSDomainHandler extends EPPDomainHandler {
 				.getMessage();
 
 		cat.debug("doDomainUpdate: command = [" + theMessage + "]");
-		
+
 		// Launch Domain Update Command?
 		if (theMessage.hasExtension(EPPLaunchUpdate.class)) {
-			EPPEventResponse launchResp = launchDomainHandler.doDomainUpdate(aEvent, aData);
-			
+			EPPEventResponse launchResp = launchDomainHandler.doDomainUpdate(
+					aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) launchResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return launchResp;
 		}
-		
-		// Related Domain Update?  
+
+		// Related Domain Update?
 		if (theMessage.hasExtension(EPPRelatedDomainExtUpdate.class)) {
-			EPPEventResponse relatedResp = relatedDomainHandler.doDomainUpdate(aEvent, aData);
-			
+			EPPEventResponse relatedResp = relatedDomainHandler.doDomainUpdate(
+					aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) relatedResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return relatedResp;
 		}
-		
+
+		// Allocation Token Update?
+		if (theMessage.hasExtension(EPPAllocationToken.class)) {
+			EPPEventResponse allocationTokenResp = allocationTokenDomainHandler
+					.doDomainUpdate(aEvent, aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) allocationTokenResp.getResponse())
+						.addExtension(theMessage
+								.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return allocationTokenResp;
+		}
+
+		// Fee Update?
+		if (theMessage
+				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeUpdate.class)
+				|| theMessage
+						.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeUpdate.class)) {
+			EPPEventResponse feeResp = feeDomainHandler.doDomainUpdate(aEvent,
+					aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) feeResp.getResponse()).addExtension(theMessage
+						.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return feeResp;
+		}
 
 		EPPTransId transId = new EPPTransId(theMessage.getTransId(), svrTransId);
 
@@ -1174,43 +1397,89 @@ public class NSDomainHandler extends EPPDomainHandler {
 	 */
 	protected EPPEventResponse doDomainInfo(EPPEvent aEvent, Object aData) {
 		EPPDomainInfoCmd theMessage = (EPPDomainInfoCmd) aEvent.getMessage();
-		
-		cat.debug("doDomainInfo: command = [" + theMessage + "]");		
-		
+
+		cat.debug("doDomainInfo: command = [" + theMessage + "]");
+
 		// Launch Domain Info Command?
 		if (theMessage.hasExtension(EPPLaunchInfo.class)) {
-			EPPEventResponse launchResp = launchDomainHandler.doDomainInfo(aEvent, aData);
-			
+			EPPEventResponse launchResp = launchDomainHandler.doDomainInfo(
+					aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) launchResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return launchResp;
 		}
-		
+
 		// Related Domain Info Command?
 		if (theMessage.hasExtension(EPPRelatedDomainExtInfo.class)) {
-			EPPEventResponse relatedDomainResp = relatedDomainHandler.doDomainInfo(aEvent, aData);
-			
+			EPPEventResponse relatedDomainResp = relatedDomainHandler
+					.doDomainInfo(aEvent, aData);
+
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
 				((EPPResponse) relatedDomainResp.getResponse())
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return relatedDomainResp;
 		}
-		
-		
+
+		// Allocation Token Info?
+		if (theMessage.hasExtension(EPPAllocationToken.class)) {
+			EPPEventResponse allocationTokenResp = allocationTokenDomainHandler
+					.doDomainInfo(aEvent, aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) allocationTokenResp.getResponse())
+						.addExtension(theMessage
+								.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return allocationTokenResp;
+		}
+
+		// Fee Info?
+		if (theMessage
+				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeInfo.class)
+				|| theMessage
+						.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeInfo.class)) {
+			EPPEventResponse feeResp = feeDomainHandler.doDomainInfo(aEvent,
+					aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) feeResp.getResponse()).addExtension(theMessage
+						.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return feeResp;
+		}
+
+		// IDN Map Info?
+		if (theMessage.getName().equalsIgnoreCase("xn--espaol-zwa.example.com")) {
+			EPPEventResponse idnMapResp = idnMapDomainHandler.doDomainInfo(aEvent,
+					aData);
+
+			// Mirror NameStore Extension in response
+			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
+				((EPPResponse) idnMapResp.getResponse()).addExtension(theMessage
+						.getExtension(EPPNamestoreExtNamestoreExt.class));
+			}
+
+			return idnMapResp;
+		}
 		
 		// .name domain?
 		if (theMessage.getName().endsWith(".name")) {
-			EPPEventResponse persRegResp = pergRegDomainHandler
-					.doDomainInfo(aEvent, aData);
+			EPPEventResponse persRegResp = pergRegDomainHandler.doDomainInfo(
+					aEvent, aData);
 
 			// Mirror NameStore Extension in response
 			if (theMessage.hasExtension(EPPNamestoreExtNamestoreExt.class)) {
@@ -1218,10 +1487,9 @@ public class NSDomainHandler extends EPPDomainHandler {
 						.addExtension(theMessage
 								.getExtension(EPPNamestoreExtNamestoreExt.class));
 			}
-			
+
 			return persRegResp;
 		}
-		
 
 		// EPPDomainInfo Response requires a vector of status
 		Vector statuses = new Vector();
@@ -1429,7 +1697,6 @@ public class NSDomainHandler extends EPPDomainHandler {
 			cat.info("NSDomainHandler.doDomainInfo: no secDNS extension URI specified in login");
 		}
 
-
 		return new EPPEventResponse(theResponse);
 	}
 }
